=== modified file 'bzrlib/_bencode_c.pyx'
--- bzrlib/_bencode_c.pyx	2007-08-12 01:53:05 +0000
+++ bzrlib/_bencode_c.pyx	2009-05-26 09:50:19 +0000
@@ -74,15 +74,16 @@
 cdef class Decoder:
     """Bencode decoder"""
 
-    cdef readonly object __s
+    cdef readonly object _text
     cdef readonly char *tail
     cdef readonly int   size
 
     cdef readonly long   _MAXINT
     cdef readonly int    _MAXN
     cdef readonly object _longint
+    cdef readonly int    _yield_tuples
 
-    def __init__(self, s):
+    def __init__(self, s, yield_tuples=0):
         """Initialize decoder engine.
         @param  s:  Python string.
         """
@@ -97,16 +98,17 @@
         if pstr == NULL:
             raise ValueError
 
-        self.__s = s
+        self._text = s
         self.tail = pstr
         self.size = <int>k
+        self._yield_tuples = int(yield_tuples)
 
         self._MAXINT = PyInt_GetMax()
         self._MAXN = len(str(self._MAXINT))
         self._longint = long(0)
 
     def __repr__(self):
-        return 'Decoder(%s)' % repr(self.__s)
+        return 'Decoder(%s)' % repr(self._text)
 
     def decode(self):
         result = self.decode_object()
@@ -226,7 +228,10 @@
         while self.size > 0:
             if self.tail[0] == SMALL_E:
                 self._update_tail(1)
-                return result
+                if self._yield_tuples:
+                    return tuple(result)
+                else:
+                    return result
             else:
                 result.append(self.decode_object())
 
@@ -263,6 +268,11 @@
     return Decoder(s).decode()
 
 
+def bdecode_as_tuple(object s):
+    """Decode string x to Python object, using tuples rather than lists."""
+    return Decoder(s, True).decode()
+
+
 class Bencached(object):
     __slots__ = ['bencoded']
 

=== modified file 'bzrlib/bencode.py'
--- bzrlib/bencode.py	2009-05-26 09:20:17 +0000
+++ bzrlib/bencode.py	2009-05-26 09:50:19 +0000
@@ -17,6 +17,6 @@
 """Wrapper around the bencode pyrex and python implementation"""
 
 try:
-    from bzrlib._bencode_c import bdecode, bencode, Bencached
+    from bzrlib._bencode_c import bdecode, bdecode_as_tuple, bencode, Bencached
 except ImportError:
-    from bzrlib._bencode_py import bdecode, bencode, Bencached
+    from bzrlib._bencode_py import bdecode, bdecode_as_tuple, bencode, Bencached

=== modified file 'bzrlib/tests/test_bencode.py'
--- bzrlib/tests/test_bencode.py	2009-05-26 09:20:17 +0000
+++ bzrlib/tests/test_bencode.py	2009-05-26 09:50:19 +0000
@@ -157,6 +157,13 @@
                             [['Alice', 'Bob'], [2, 3]])
                         ])
 
+    def test_list_as_tuple(self):
+        self._run_check([('le', ()),
+                         ('li1ei2ei3ee', (1, 2, 3)),
+                         ('ll5:Alice3:Bobeli2ei3eee',
+                            (('Alice', 'Bob'), (2, 3)))
+                        ])
+
     def test_dict(self):
         self._run_check([('de', {}),
                          ('d3:agei25e4:eyes4:bluee',

